package biz

import (
	"context"
	"time"

	"github.com/go-kratos/kratos/v2/log"

	"demo/internal/biz/common"
	"demo/internal/biz/errs"
	"demo/internal/util"
)

type TimeGranularity string

const (
	StatTimeGranularityDaily  TimeGranularity = "STAT_TIME_GRANULARITY_DAILY"  // 按天维度
	StatTimeGranularityHourly TimeGranularity = "STAT_TIME_GRANULARITY_HOURLY" // 按小时维度
)

type ListReportRequest struct {
	common.PageRequest
	TimeGranularity TimeGranularity `form:"time_granularity" json:"time_granularity"` // 时间粒度(允许值:STAT_TIME_GRANULARITY_DAILY (按天维度),STAT_TIME_GRANULARITY_HOURLY (按小时维度))
	StartDate       string          `form:"start_date" json:"start_date"`             // 开始时间(格式YYYY-MM-DD)
	EndDate         string          `form:"end_date" json:"end_date"`                 // 结束时间(格式YYYY-MM-DD)
	StartDateLong   util.Second     `json:"-"`                                        // 开始时间
	EndDateLong     util.Second     `json:"-"`                                        // 结束时间
}

func (p *ListReportRequest) Validate() error {
	// set default data today
	today := time.Now().Format("2006-01-02")
	if p.StartDate == "" {
		p.StartDate = today
	}
	if p.EndDate == "" {
		p.EndDate = today
	}
	parse, err := time.Parse("2006-01-02", p.StartDate)
	if err != nil {
		return errs.ErrorArgs.WithMessage("开始时间格式错误")
	}
	p.StartDateLong = util.NewSecond(parse)

	parse, err = time.Parse("2006-01-02", p.EndDate)
	if err != nil {
		return errs.ErrorArgs.WithMessage("结束时间格式错误")
	}
	p.EndDateLong = util.NewSecond(parse)
	return nil
}

type ListCampaignResponse struct {
	List     CampaignList     `json:"list"`      // 数据信息
	PageInfo *common.PageInfo `json:"page_info"` // 页码信息
}

type CampaignList []*Campaign

func (p CampaignList) PrepareRateAndCost() {
	for _, campaign := range p {
		campaign.PrepareRateAndCost()
	}
}

type Campaign struct {
	CampaignId                            string         `json:"campaign_id"`                                 // 广告组ID
	CampaignName                          string         `json:"campaign_name"`                               // 广告主组名称
	AdvertiserId                          string         `json:"advertiser_id"`                               // 广告主id
	StatDatetime                          util.Second    `json:"stat_datetime"`                               // 数据起始时间(yyyy-MM-dd HH:mm:ss)
	Cost                                  float64        `json:"cost"`                                        // 花费
	Show                                  int            `json:"show"`                                        // 展示量
	Click                                 int            `json:"click"`                                       // 点击量
	Convert                               int            `json:"convert"`                                     // 转化数
	DeepConvert                           int            `json:"deep_convert"`                                // 深度转化数
	AttributionConvert                    int            `json:"attribution_convert"`                         // 转化数（计费时间）
	AttributionDeepConvert                int            `json:"attribution_deep_convert"`                    // 深度转化数（计费时间）
	DownloadStart                         int            `json:"download_start"`                              // 安卓下载开始数
	DownloadFinish                        int            `json:"download_finish"`                             // 安卓下载完成数
	ClickInstall                          int            `json:"click_install"`                               // 点击安装数
	InstallFinish                         int            `json:"install_finish"`                              // 安卓安装完成数
	Active                                int            `json:"active"`                                      // 激活数
	Register                              int            `json:"register"`                                    // 注册数
	NextDayOpen                           int            `json:"next_day_open"`                               // 次留数
	NextDayOpenCost                       float64        `json:"next_day_open_cost"`                          // 次留成本
	NextDayOpenRate                       float64        `json:"next_day_open_rate"`                          // 次留率
	AttributionNextDayOpenCnt             int            `json:"attribution_next_day_open_cnt"`               // 次留回传数
	GameAddiction                         int            `json:"game_addiction"`                              // 关键行为数
	PayCount                              int            `json:"pay_count"`                                   // 首次付费次数
	InAppUv                               int            `json:"in_app_uv"`                                   // APP内访问
	InAppDetailUv                         int            `json:"in_app_detail_uv"`                            // APP内访问详情页
	InAppCart                             int            `json:"in_app_cart"`                                 // APP内加入购物车
	InAppPay                              int            `json:"in_app_pay"`                                  // APP内付费
	InAppOrder                            int            `json:"in_app_order"`                                // APP内下单
	AttributionGamePay7DCount             int            `json:"attribution_game_pay_7d_count"`               // 7日付费次数（激活时间）
	GamePayCount                          int            `json:"game_pay_count"`                              // 付费次数
	TotalPlay                             int            `json:"total_play"`                                  // 视频数据-播放数
	ValidPlay                             int            `json:"valid_play"`                                  // 视频数据-有效播放数
	Play25FeedBreak                       int            `json:"play_25_feed_break"`                          // 视频数据-25%进度播放数
	Play50FeedBreak                       int            `json:"play_50_feed_break"`                          // 视频数据-50%进度播放数
	Play75FeedBreak                       int            `json:"play_75_feed_break"`                          // 视频数据-75%进度播放数
	Play100FeedBreak                      int            `json:"play_100_feed_break"`                         // 视频数据-99%进度播放数
	PlayDurationSum                       int            `json:"play_duration_sum"`                           // 视频数据-播放时长，单位ms
	AttributionActivePayIntraOneDayCount  int            `json:"attribution_active_pay_intra_one_day_count"`  // 游戏行业-激活后24h首次付费数
	AttributionActivePayIntraOneDayAmount float64        `json:"attribution_active_pay_intra_one_day_amount"` // 游戏行业-激活后24h付费金额
	Timestamp                             util.CivilTime `json:"timestamp"`                                   // 创建时间(yyyy-MM-dd HH:mm:ss)
	// 计算字段
	AvgShowCost                util.Float64 `json:"avg_show_cost"`                  // 千次展示成本
	AvgClickCost               util.Float64 `json:"avg_click_cost"`                 // 点击成本
	Ctr                        util.Float64 `json:"ctr"`                            // 点击率
	ConvertCost                util.Float64 `json:"convert_cost"`                   // 转化成本
	ConvertRate                util.Float64 `json:"convert_rate"`                   // 转化率
	DeepConvertCost            util.Float64 `json:"deep_convert_cost"`              // 深度转化成本
	DeepConvertRate            util.Float64 `json:"deep_convert_rate"`              // 深度转化率
	AttributionConvertCost     util.Float64 `json:"attribution_convert_cost"`       // 转化成本（计费时间）
	AttributionDeepConvertCost util.Float64 `json:"attribution_deep_convert_cost"`  // 深度转化成本（计费时间）
	DownloadStartCost          util.Float64 `json:"download_start_cost"`            // 安卓下载开始成本
	DownloadStartRate          util.Float64 `json:"download_start_rate"`            // 安卓下载开始率
	DownloadFinishCost         util.Float64 `json:"download_finish_cost"`           // 安卓下载完成成本
	DownloadFinishRate         util.Float64 `json:"download_finish_rate"`           // 安卓下载完成率
	InstallFinishCost          util.Float64 `json:"install_finish_cost"`            // 安卓安装完成成本
	InstallFinishRate          util.Float64 `json:"install_finish_rate"`            // 安卓安装完成率
	ActiveCost                 util.Float64 `json:"active_cost"`                    // 激活成本
	ActiveRate                 util.Float64 `json:"active_rate"`                    // 激活率
	ActiveRegisterCost         util.Float64 `json:"active_register_cost"`           // 注册成本
	ActiveRegisterRate         util.Float64 `json:"active_register_rate"`           // 注册率
	AttributionNextDayOpenCost util.Float64 `json:"attribution_next_day_open_cost"` // 次留成本
	AttributionNextDayOpenRate util.Float64 `json:"attribution_next_day_open_rate"` // 次留率
	GameAddictionCost          util.Float64 `json:"game_addiction_cost"`            // 关键行为成本
	GameAddictionRate          util.Float64 `json:"game_addiction_rate"`            // 关键行为率
	ActivePayCost              util.Float64 `json:"active_pay_cost"`                // 首次付费成本
	ActivePayRate              util.Float64 `json:"active_pay_rate"`                // 首次付费率
	ValidPlayCost              util.Float64 `json:"valid_play_cost"`                // 有效播放成本
	ValidPlayRate              util.Float64 `json:"valid_play_rate"`                // 有效播放率
	AveragePlayTimePerPlay     util.Float64 `json:"average_play_time_per_play"`     // 视频数据-平均单次播放时长，单位：秒
	PlayOverRate               util.Float64 `json:"play_over_rate"`                 // 播完率
}

func (p *Campaign) PrepareRateAndCost() {
	p.AvgShowCost = p.getAvgShowCost()
	p.AvgClickCost = p.getAvgClickCost()
	p.Ctr = p.getCtr()
	p.ConvertCost = p.getConvertCost()
	p.ConvertRate = p.getConvertRate()
	p.DeepConvertCost = p.getDeepConvertCost()
	p.DeepConvertRate = p.getDeepConvertRate()
	p.AttributionConvertCost = p.getAttributionConvertCost()
	p.AttributionDeepConvertCost = p.getAttributionDeepConvertCost()
	p.DownloadStartCost = p.getDownloadStartCost()
	p.DownloadStartRate = p.getDownloadStartRate()
	p.DownloadFinishCost = p.getDownloadFinishCost()
	p.DownloadFinishRate = p.getDownloadFinishRate()
	p.InstallFinishCost = p.getInstallFinishCost()
	p.InstallFinishRate = p.getInstallFinishRate()
	p.ActiveCost = p.getActiveCost()
	p.ActiveRate = p.getActiveRate()
	p.ActiveRegisterCost = p.getActiveRegisterCost()
	p.ActiveRegisterRate = p.getActiveRegisterRate()
	p.AttributionNextDayOpenCost = p.getAttributionNextDayOpenCost()
	p.AttributionNextDayOpenRate = p.getAttributionNextDayOpenRate()
	p.GameAddictionCost = p.getGameAddictionCost()
	p.GameAddictionRate = p.getGameAddictionRate()
	p.ActivePayCost = p.getActivePayCost()
	p.ActivePayRate = p.getActivePayRate()
	p.ValidPlayCost = p.getValidPlayCost()
	p.ValidPlayRate = p.getValidPlayRate()
	p.AveragePlayTimePerPlay = p.getAveragePlayTimePerPlay()
	p.PlayOverRate = p.getPlayOverRate()
}

func (p *Campaign) getAvgShowCost() util.Float64 {
	return calcRate(p.Cost, float64(p.Show)) * 1000
}

func (p *Campaign) getAvgClickCost() util.Float64 {
	return calcRate(p.Cost, float64(p.Click))
}

func (p *Campaign) getCtr() util.Float64 {
	return calcRate(float64(p.Click), float64(p.Show)) * 100
}

func (p *Campaign) getConvertCost() util.Float64 {
	return calcRate(p.Cost, float64(p.Convert))
}

func (p *Campaign) getConvertRate() util.Float64 {
	return calcRate(float64(p.Convert), float64(p.Click)) * 100
}

func (p *Campaign) getDeepConvertCost() util.Float64 {
	return calcRate(p.Cost, float64(p.DeepConvert))
}

func (p *Campaign) getDeepConvertRate() util.Float64 {
	return calcRate(float64(p.DeepConvert), float64(p.Convert)) * 100
}

func (p *Campaign) getAttributionConvertCost() util.Float64 {
	return calcRate(p.Cost, float64(p.AttributionConvert))
}

func (p *Campaign) getAttributionDeepConvertCost() util.Float64 {
	return calcRate(p.Cost, float64(p.AttributionDeepConvert))
}

func (p *Campaign) getDownloadStartCost() util.Float64 {
	return calcRate(p.Cost, float64(p.DownloadStart))
}

func (p *Campaign) getDownloadStartRate() util.Float64 {
	return calcRate(float64(p.DownloadStart), float64(p.Click)) * 100
}

func (p *Campaign) getDownloadFinishCost() util.Float64 {
	return calcRate(p.Cost, float64(p.DownloadFinish))
}

func (p *Campaign) getDownloadFinishRate() util.Float64 {
	return calcRate(float64(p.DownloadFinish), float64(p.DownloadStart)) * 100
}

func (p *Campaign) getInstallFinishCost() util.Float64 {
	return calcRate(p.Cost, float64(p.InstallFinish))
}

func (p *Campaign) getInstallFinishRate() util.Float64 {
	return calcRate(float64(p.InstallFinish), float64(p.DownloadFinish)) * 100
}

func (p *Campaign) getActiveCost() util.Float64 {
	return calcRate(p.Cost, float64(p.Active))
}

func (p *Campaign) getActiveRate() util.Float64 {
	return calcRate(float64(p.Active), float64(p.Click)) * 100
}

func (p *Campaign) getActiveRegisterCost() util.Float64 {
	return calcRate(p.Cost, float64(p.Register))
}

func (p *Campaign) getActiveRegisterRate() util.Float64 {
	return calcRate(float64(p.Register), float64(p.Active)) * 100
}

func (p *Campaign) getAttributionNextDayOpenCost() util.Float64 {
	return calcRate(p.Cost, float64(p.AttributionNextDayOpenCnt))
}

func (p *Campaign) getAttributionNextDayOpenRate() util.Float64 {
	return calcRate(float64(p.AttributionNextDayOpenCnt), float64(p.Active)) * 100
}

func (p *Campaign) getGameAddictionCost() util.Float64 {
	return calcRate(p.Cost, float64(p.GameAddiction))
}

func (p *Campaign) getGameAddictionRate() util.Float64 {
	return calcRate(float64(p.GameAddiction), float64(p.Active)) * 100
}

func (p *Campaign) getActivePayCost() util.Float64 {
	return calcRate(p.Cost, float64(p.PayCount))
}

func (p *Campaign) getActivePayRate() util.Float64 {
	return calcRate(float64(p.PayCount), float64(p.Active)) * 100
}

func (p *Campaign) getValidPlayCost() util.Float64 {
	return calcRate(p.Cost, float64(p.ValidPlay))
}

func (p *Campaign) getValidPlayRate() util.Float64 {
	return calcRate(float64(p.ValidPlay), float64(p.Show)) * 100
}

func (p *Campaign) getAveragePlayTimePerPlay() util.Float64 {
	return calcRate(float64(p.PlayDurationSum), float64(p.TotalPlay)) / 1000
}

func (p *Campaign) getPlayOverRate() util.Float64 {
	return calcRate(float64(p.Play100FeedBreak), float64(p.TotalPlay)) * 100
}

func calcRate(x float64, y float64) util.Float64 {
	if y == 0 {
		return 0
	}
	return util.Float64(x / y)
}

// CampaignRepo is a Report camRepo.
type CampaignRepo interface {
	List(ctx context.Context, customerId int64, req *ListReportRequest) (int64, CampaignList, error)
}

type ListAdResponse struct {
	List     AdList           `json:"list"`      // 数据信息
	PageInfo *common.PageInfo `json:"page_info"` // 页码信息
}

type AdList []*Ad

func (p AdList) PrepareRateAndCost() {
	for _, ad := range p {
		ad.PrepareRateAndCost()
	}
}

type Ad struct {
	AdId   string `json:"ad_id"`   // 广告计划id
	AdName string `json:"ad_name"` // 广告计划名称
	Campaign
}

type AdRepo interface {
	List(ctx context.Context, customerId int64, req *ListReportRequest) (int64, AdList, error)
}

type ListCreativeResponse struct {
	List     CreativeList     `json:"list"`      // 数据信息
	PageInfo *common.PageInfo `json:"page_info"` // 页码信息
}

type CreativeList []*Creative

func (p CreativeList) PrepareRateAndCost() {
	for _, ad := range p {
		ad.PrepareRateAndCost()
	}
}

type Creative struct {
	CreativeId string `json:"creative_id"` // 创意id
	Ad
}

type CreativeRepo interface {
	List(ctx context.Context, customerId int64, req *ListReportRequest) (int64, CreativeList, error)
}

// ReportUsecase is a Report usecase.
type ReportUsecase struct {
	camRepo      CampaignRepo
	adRepo       AdRepo
	creativeRepo CreativeRepo
	log          *log.Helper
}

// NewReportUsecase new a Report usecase.
func NewReportUsecase(repo CampaignRepo, adRepo AdRepo, creativeRepo CreativeRepo, logger log.Logger) *ReportUsecase {
	return &ReportUsecase{camRepo: repo, adRepo: adRepo, creativeRepo: creativeRepo, log: log.NewHelper(logger)}
}

// ListCampaignData list campaign data.
func (p *ReportUsecase) ListCampaignData(ctx context.Context, customerId int64, req *ListReportRequest) (*ListCampaignResponse, error) {
	p.log.WithContext(ctx).Infof("req: %v", req)
	total, list, err := p.camRepo.List(ctx, customerId, req)
	if err != nil {
		return nil, err
	}
	list.PrepareRateAndCost()
	return &ListCampaignResponse{
		List:     list,
		PageInfo: req.PageInfo(total),
	}, nil
}

// ListAdData list ad data.
func (p *ReportUsecase) ListAdData(ctx context.Context, customerId int64, req *ListReportRequest) (*ListAdResponse, error) {
	p.log.WithContext(ctx).Infof("req: %v", req)
	total, list, err := p.adRepo.List(ctx, customerId, req)
	if err != nil {
		return nil, err
	}
	list.PrepareRateAndCost()
	return &ListAdResponse{
		List:     list,
		PageInfo: req.PageInfo(total),
	}, nil
}

// ListCreativeData list creative data.
func (p *ReportUsecase) ListCreativeData(ctx context.Context, customerId int64, req *ListReportRequest) (*ListCreativeResponse, error) {
	p.log.WithContext(ctx).Infof("req: %v", req)
	total, list, err := p.creativeRepo.List(ctx, customerId, req)
	if err != nil {
		return nil, err
	}
	list.PrepareRateAndCost()
	return &ListCreativeResponse{
		List:     list,
		PageInfo: req.PageInfo(total),
	}, nil
}
